# Standard libraries:
from typing import TypeAlias, Optional, Tuple, Dict, Any
from tkinter import Widget, Event, Misc


class TransparentMisc(Misc):
    is_transparent: bool = False

    def __init__(self, master: Optional[Misc] = None, *, background: Optional[str] = "transparent", **standard_options: Any) -> None:
        assert isinstance(self, Widget), "This instance must be a Widget."
        super().__init__(master, **standard_options)
        self.configure(background=background)
        self.bind(sequence="<<ParentConfigure>>", func=self.on_parent_configure, add=True)

    def configure(self, **standard_options: Any) -> Any:
        background_color: Optional[str] = standard_options.pop("background", None)
        background_color = standard_options.pop("bg", background_color)
        if background_color is not None:
            self.is_transparent = isinstance(background_color, str) and background_color == "transparent"
            if not self.is_transparent:
                standard_options["background"] = background_color
            self.on_parent_configure()
        return super().configure(**standard_options)
    config = configure

    def on_parent_configure(self, event: Optional[Event] = None) -> None:
        if not self.is_transparent:
            return None
        parent_background_color: Optional[str] = Misc.cget(self.master, key="background") or None
        if parent_background_color is None:
            return None
        Misc.configure(self.master, background=parent_background_color)
        for child_widget in self.winfo_children():
            child_widget.event_generate(sequence="<<ParentConfigure>>")



# Standard libraries:
from setuptools import setup, find_packages
from typing import Dict, List, Any

# Commands:
# pip install twine wheel
# python setup.py sdist bdist_wheel
# twine upload dist/*

# Setup:
application_packages: List[str] = find_packages()
application_settings: Dict[str, Any] = {
    "name": "TkEnhanced",
    "version": "1.0.1",
    "description": "Enhanced Tkinter widgets for a modern look and additional functionality.",
    "author": "Bunnitown",
    "packages": application_packages,
    "install_requires": [],
    "classifiers": [
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.11",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ]}
setup(**application_settings)



# Local imports:
from . import EnhFrame

# Standard libraries:
from typing import Optional, Any
from tkinter import Misc


class EnhScrollableFrame(EnhFrame):
    def __init__(self, master: Optional[Misc] = None, *, background: Optional[str] = "transparent", **standard_options: Any) -> None:
        super().__init__(master, background=background, **standard_options)
